@@ -3,6 +3,7 @@ from lintian_brush import USER_AGENT
 from lintian_brush.control import (
     update_control,
     )
+import os
 import sys
 import urllib.error
 import urllib.parse
@@ -13,6 +14,8 @@ known_https = [
     'pear.php.net', 'pecl.php.net', 'www.bioconductor.org',
     'cran.r-project.org', 'wiki.debian.org']
 
+ERRORS = (urllib.error.URLError, urllib.error.HTTPError, ConnectionResetError)
+
 
 def same_page(http_contents, https_contents):
     # This is a pretty crude way to determine we end up on the same page, but
@@ -30,12 +33,20 @@ def fix_homepage(http_url):
     result = urllib.parse.urlparse(http_url)
     if result.netloc in known_https:
         return https_url
+    if os.environ.get('NET_ACCESS', 'allow') != 'allow':
+        return http_url
     # Fall back to just comparing the two
     headers = {'User-Agent': USER_AGENT}
-    http_contents = urlopen(Request(http_url, headers=headers)).read()
+    try:
+        http_contents = urlopen(Request(http_url, headers=headers)).read()
+    except ERRORS as e:
+        sys.stderr.write(
+            'Unable to access HTTP version of homepage %s: %s' %
+            (http_url, e))
+        return http_url
     try:
         https_contents = urlopen(Request(https_url, headers=headers)).read()
-    except urllib.error.URLError as e:
+    except ERRORS as e:
         sys.stderr.write(
             'Unable to access HTTPS version of homepage %s: %s' %
             (https_url, e))
