@@ -40,13 +40,16 @@ from breezy.clean_tree import (
 from breezy.commit import NullCommitReporter
 from breezy.errors import NoSuchFile
 from breezy.rename_map import RenameMap
-from breezy.trace import note
+from breezy.trace import note, warning
 from breezy.transform import revert
 
 from debian.deb822 import Deb822
 
 
-__version__ = (0, 26)
+from .reformatting import FormattingUnpreservable
+
+
+__version__ = (0, 38)
 version_string = '.'.join(map(str, __version__))
 SUPPORTED_CERTAINTIES = ['certain', 'possible', None]
 DEFAULT_MINIMUM_CERTAINTY = 'certain'
@@ -99,8 +102,8 @@ class DescriptionMissing(Exception):
 class NotDebianPackage(Exception):
     """The specified directory does not contain a Debian package."""
 
-    def __init__(self, tree):
-        super(NotDebianPackage, self).__init__(tree.basedir)
+    def __init__(self, tree, path):
+        super(NotDebianPackage, self).__init__(tree.abspath(path))
 
 
 class PendingChanges(Exception):
@@ -201,13 +204,15 @@ class PythonScriptFixer(Fixer):
 
     def run(self, basedir, current_version, compat_release,
             minimum_certainty=DEFAULT_MINIMUM_CERTAINTY,
-            trust_package=False, allow_reformatting=False):
+            trust_package=False, allow_reformatting=False,
+            net_access=True):
         env = dict(os.environ.items())
         env['CURRENT_VERSION'] = str(current_version)
         env['COMPAT_RELEASE'] = compat_release
         env['MINIMUM_CERTAINTY'] = minimum_certainty
         env['TRUST_PACKAGE'] = 'true' if trust_package else 'false'
         env['REFORMATTING'] = ('allow' if allow_reformatting else 'disallow')
+        env['NET_ACCESS'] = ('allow' if net_access else 'disallow')
         try:
             old_env = os.environ
             old_stderr = sys.stderr
@@ -221,6 +226,8 @@ class PythonScriptFixer(Fixer):
                 with open(self.script_path, 'r') as f:
                     code = compile(f.read(), self.script_path, 'exec')
                     exec(code, {})
+            except FormattingUnpreservable:
+                raise
             except SystemExit as e:
                 retcode = e.code
             except BaseException as e:
@@ -259,13 +266,15 @@ class ScriptFixer(Fixer):
 
     def run(self, basedir, current_version, compat_release,
             minimum_certainty=DEFAULT_MINIMUM_CERTAINTY,
-            trust_package=False, allow_reformatting=False):
+            trust_package=False, allow_reformatting=False,
+            net_access=True):
         env = dict(os.environ.items())
         env['CURRENT_VERSION'] = str(current_version)
         env['COMPAT_RELEASE'] = compat_release
         env['MINIMUM_CERTAINTY'] = minimum_certainty
         env['TRUST_PACKAGE'] = 'true' if trust_package else 'false'
         env['REFORMATTING'] = ('allow' if allow_reformatting else 'disallow')
+        env['NET_ACCESS'] = ('allow' if net_access else 'disallow')
         with tempfile.SpooledTemporaryFile() as stderr:
             p = subprocess.Popen(self.script_path, cwd=basedir,
                                  stdout=subprocess.PIPE, stderr=stderr,
@@ -320,6 +329,24 @@ def read_desc_file(path):
                 yield ScriptFixer(name, tags, script_path)
 
 
+def select_fixers(fixers, names):
+    """Select fixers by name, from a list.
+
+    Args:
+      fixers: List of Fixer objects
+      names: List of names to select
+    Raises:
+      KeyError: if one of the names did not exist
+    """
+    names = set(names)
+    available = set([f.name for f in fixers])
+    missing = names - available
+    if missing:
+        raise KeyError(missing.pop())
+    # Preserve order
+    return [f for f in fixers if f.name in names]
+
+
 def available_lintian_fixers(fixers_dir=None):
     """Return a list of available lintian fixers.
 
@@ -414,11 +441,12 @@ def get_committer(tree):
         return config.get('email')
 
 
-def only_changes_last_changelog_block(tree):
+def only_changes_last_changelog_block(tree, changelog_path):
     """Check whether the only change in a tree is to the last changelog entry.
 
     Args:
       tree: Tree to analyze
+      changelog_path: Path to the changelog file
     Returns:
       boolean
     """
@@ -435,10 +463,14 @@ def only_changes_last_changelog_block(tree):
             pass
         else:
             return False
-        if first_change[1] != ('debian/changelog', 'debian/changelog'):
+        try:
+            change_paths = first_change.path
+        except AttributeError:  # brz < 3.1
+            change_paths = first_change[1]
+        if change_paths != (changelog_path, changelog_path):
             return False
-        new_cl = Changelog(tree.get_file_text('debian/changelog'))
-        old_cl = Changelog(basis_tree.get_file_text('debian/changelog'))
+        new_cl = Changelog(tree.get_file_text(changelog_path))
+        old_cl = Changelog(basis_tree.get_file_text(changelog_path))
         if old_cl.distributions != "UNRELEASED":
             return False
         del new_cl._blocks[0]
@@ -446,7 +478,7 @@ def only_changes_last_changelog_block(tree):
         return str(new_cl) == str(old_cl)
 
 
-def reset_tree(local_tree, dirty_tracker=None):
+def reset_tree(local_tree, dirty_tracker=None, subpath='.'):
     """Reset a tree back to its basis tree.
 
     This will leave ignored and detritus files alone.
@@ -457,7 +489,8 @@ def reset_tree(local_tree, dirty_tracker=None):
     """
     if dirty_tracker and not dirty_tracker.is_dirty():
         return
-    revert(local_tree, local_tree.branch.basis_tree(), None)
+    revert(local_tree, local_tree.branch.basis_tree(),
+           [subpath] if subpath != '.' else None)
     deletables = list(iter_deletables(
         local_tree, unknown=True, ignored=False, detritus=False))
     delete_items(deletables)
@@ -492,10 +525,35 @@ def check_clean_tree(local_tree):
         raise PendingChanges(local_tree)
 
 
+def add_changelog_entry(tree, path, summary):
+    """Add a changelog entry.
+
+    Args:
+      tree: Tree to edit
+      path: Path to the changelog file
+      summary: Entry to add
+    """
+    p = subprocess.Popen(
+        ["dch", "--no-auto-nmu", summary],
+        cwd=tree.abspath(os.path.dirname(os.path.dirname(path))),
+        stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE)
+    (stdout, stderr) = p.communicate(b"\n")
+    if stdout:
+        raise ValueError(stdout)
+    if stderr:
+        end = b'dch: Did you see that warning?  Press RETURN to continue...\n'
+        if stderr.endswith(end):
+            stderr = stderr[:-len(end)]
+        for line in stderr.splitlines():
+            warning('%s', line)
+
+
 def run_lintian_fixer(local_tree, fixer, committer=None,
                       update_changelog=None, compat_release=None,
                       minimum_certainty=None, trust_package=False,
-                      allow_reformatting=False, dirty_tracker=None):
+                      allow_reformatting=False, dirty_tracker=None,
+                      subpath='.', net_access=True):
     """Run a lintian fixer on a tree.
 
     Args:
@@ -511,14 +569,21 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
       allow_reformatting: Whether to allow reformatting of changed files
       dirty_tracker: Optional object that can be used to tell if the tree
         has been changed.
+      subpath: Path in tree to operate on
+      net_access: Whether to allow accessing external services
     Returns:
       tuple with set of FixerResult, summary of the changes
     """
+    if subpath == '.':
+        changelog_path = 'debian/changelog'
+    else:
+        changelog_path = os.path.join(subpath, 'debian/changelog')
+
     try:
-        with local_tree.get_file('debian/changelog') as f:
+        with local_tree.get_file(changelog_path) as f:
             cl = Changelog(f, max_blocks=1)
     except NoSuchFile:
-        raise NotDebianPackage(local_tree)
+        raise NotDebianPackage(local_tree, subpath)
     if cl.distributions == 'UNRELEASED':
         current_version = cl.version
     else:
@@ -530,16 +595,18 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
         minimum_certainty = DEFAULT_MINIMUM_CERTAINTY
     try:
         result = fixer.run(
-            local_tree.basedir, current_version=current_version,
+            local_tree.abspath(subpath),
+            current_version=current_version,
             compat_release=compat_release,
             minimum_certainty=minimum_certainty,
             trust_package=trust_package,
-            allow_reformatting=allow_reformatting)
+            allow_reformatting=allow_reformatting,
+            net_access=net_access)
     except BaseException:
-        reset_tree(local_tree, dirty_tracker)
+        reset_tree(local_tree, dirty_tracker, subpath)
         raise
     if not certainty_sufficient(result.certainty, minimum_certainty):
-        reset_tree(local_tree, dirty_tracker)
+        reset_tree(local_tree, dirty_tracker, subpath)
         raise NoChanges("Certainty of script's changes not high enough")
     if dirty_tracker:
         relpaths = dirty_tracker.relpaths()
@@ -555,8 +622,8 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
         if not specific_files:
             raise NoChanges("Script didn't make any changes")
     else:
-        local_tree.smart_add([local_tree.basedir])
-        specific_files = None
+        local_tree.smart_add([local_tree.abspath(subpath)])
+        specific_files = [subpath] if subpath != '.' else None
 
     basis_tree = local_tree.basis_tree()
     if local_tree.supports_setting_file_ids():
@@ -579,16 +646,16 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
         # Default to true. Perhaps do something more clever.
         update_changelog = True
 
-    if update_changelog and only_changes_last_changelog_block(local_tree):
+    if update_changelog and only_changes_last_changelog_block(
+            local_tree, changelog_path):
         # If the script only changed the last entry in the changelog,
         # don't update the changelog
         update_changelog = False
 
     if update_changelog:
-        subprocess.check_call(
-            ["dch", "--no-auto-nmu", summary], cwd=local_tree.basedir)
+        add_changelog_entry(local_tree, changelog_path, summary)
         if specific_files:
-            specific_files.append('debian/changelog')
+            specific_files.append(changelog_path)
 
     description = result.description
     for tag in result.fixed_lintian_tags:
@@ -609,11 +676,25 @@ def run_lintian_fixer(local_tree, fixer, committer=None,
     return result, summary
 
 
+class ManyResult(object):
+
+    def __init__(self):
+        self.success = []
+        self.failed_fixers = {}
+        self.formatting_unpreservable = {}
+
+    def __tuple__(self):
+        return (self.success, self.failed_fixers)
+
+    def __iter__(self):
+        return iter(self.__tuple__())
+
+
 def run_lintian_fixers(local_tree, fixers, update_changelog=True,
                        verbose=False, committer=None,
                        compat_release=None, minimum_certainty=None,
                        trust_package=False, allow_reformatting=False,
-                       use_inotify=None):
+                       use_inotify=None, subpath='.', net_access=True):
     """Run a set of lintian fixers on a tree.
 
     Args:
@@ -630,6 +711,8 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
       allow_reformatting: Whether to allow reformatting of changed files
       use_inotify: Use inotify to watch changes (significantly improves
         performance). Defaults to None (automatic)
+      subpath: Subpath in the tree in which the package lives
+      net_access: Whether to allow network access
     Returns:
       Tuple with two lists:
         1. list of tuples with (lintian-tag, certainty, description) of fixers
@@ -638,12 +721,10 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
            error that occurred
     """
     check_clean_tree(local_tree)
-    failed_fixers = {}
     fixers = list(fixers)
-    ret = []
     if use_inotify is True:
         from .dirty_tracker import DirtyTracker
-        dirty_tracker = DirtyTracker(local_tree)
+        dirty_tracker = DirtyTracker(local_tree, subpath)
     elif use_inotify is False:
         dirty_tracker = None
     else:
@@ -652,10 +733,12 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
         except ImportError:
             dirty_tracker = None
         else:
-            dirty_tracker = DirtyTracker(local_tree)
+            dirty_tracker = DirtyTracker(local_tree, subpath)
+    ret = ManyResult()
     with ui.ui_factory.nested_progress_bar() as pb:
         for i, fixer in enumerate(fixers):
-            pb.update('Running fixer %r on %s' % (fixer, local_tree.basedir),
+            pb.update('Running fixer %r on %s' %
+                      (fixer, local_tree.abspath(subpath)),
                       i, len(fixers))
             start = time.time()
             if dirty_tracker:
@@ -667,9 +750,15 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
                         minimum_certainty=minimum_certainty,
                         trust_package=trust_package,
                         allow_reformatting=allow_reformatting,
-                        dirty_tracker=dirty_tracker)
+                        dirty_tracker=dirty_tracker,
+                        subpath=subpath, net_access=net_access)
+            except FormattingUnpreservable as e:
+                ret.formatting_unpreservable[fixer.name] = e.path
+                if verbose:
+                    note('Fixer %r was unable to preserve formatting of %s.',
+                         fixer.name, e.path)
             except FixerFailed as e:
-                failed_fixers[fixer.name] = e
+                ret.failed_fixers[fixer.name] = e
                 if verbose:
                     note('Fixer %r failed to run.', fixer.name)
                     sys.stderr.write(str(e))
@@ -681,8 +770,8 @@ def run_lintian_fixers(local_tree, fixers, update_changelog=True,
                 if verbose:
                     note('Fixer %r made changes. (took %.2fs)',
                          fixer.name, time.time() - start)
-                ret.append((result, summary))
-    return ret, failed_fixers
+                ret.success.append((result, summary))
+    return ret
 
 
 def certainty_to_confidence(certainty):
