@@ -41,7 +41,7 @@ breezy.initialize()
 import breezy.git  # noqa: E402
 import breezy.bzr  # noqa: E402
 
-from breezy.trace import note  # noqa: E402
+from breezy.trace import note, show_error  # noqa: E402
 
 from . import (  # noqa: E402
     NotDebianPackage,
@@ -50,10 +50,12 @@ from . import (  # noqa: E402
     find_fixers_dir,
     get_committer,
     run_lintian_fixers,
+    select_fixers,
     version_string,
     SUPPORTED_CERTAINTIES,
     DEFAULT_MINIMUM_CERTAINTY,
     )
+from .config import Config   # noqa: E402
 
 
 def main(argv=None):
@@ -95,19 +97,23 @@ def main(argv=None):
         '--identity',
         help='Print user identity that would be used when committing',
         action='store_true', default=False)
+    parser.add_argument(
+        '--disable-net-access',
+        help='Do not probe external services.',
+        action='store_true', default=False)
     # Hide the minimum-certainty option for the moment.
     parser.add_argument(
         '--minimum-certainty',
         type=str,
         choices=SUPPORTED_CERTAINTIES,
-        default=DEFAULT_MINIMUM_CERTAINTY,
+        default=None,
         help=argparse.SUPPRESS)
     parser.add_argument(
         '--trust',
         action='store_true',
         help=argparse.SUPPRESS)
     parser.add_argument(
-        '--allow-reformatting', default=False, action='store_true',
+        '--allow-reformatting', default=None, action='store_true',
         help=argparse.SUPPRESS)
     parser.add_argument(
         '--disable-inotify', action='store_true', default=False,
@@ -134,7 +140,7 @@ def main(argv=None):
     else:
         try:
             if args.dry_run:
-                branch = Branch.open(args.directory)
+                branch, subpath = Branch.open_containing(args.directory)
                 td = tempfile.mkdtemp()
                 atexit.register(shutil.rmtree, td)
                 # TODO(jelmer): Make a slimmer copy
@@ -144,7 +150,7 @@ def main(argv=None):
                     stacked=branch._format.supports_stacking())
                 wt = to_dir.open_workingtree()
             else:
-                wt = WorkingTree.open(args.directory)
+                wt, subpath = WorkingTree.open_containing(args.directory)
         except NotBranchError:
             note('No version control directory found (e.g. a .git directory).')
             return 1
@@ -157,32 +163,63 @@ def main(argv=None):
             return 0
         since_revid = wt.last_revision()
         if args.fixers:
-            fixers = [f for f in fixers if f.name in args.fixers]
+            try:
+                fixers = select_fixers(fixers, args.fixers)
+            except KeyError as e:
+                show_error('Unknown fixer specified: %s', e.args[0])
+                return 1
         debian_info = distro_info.DebianDistroInfo()
         if args.modern:
             compat_release = debian_info.devel()
         else:
+            compat_release = None
+        minimum_certainty = args.minimum_certainty
+        allow_reformatting = args.allow_reformatting
+        try:
+            cfg = Config.from_workingtree(wt, subpath)
+        except FileNotFoundError:
+            pass
+        else:
+            if minimum_certainty is None:
+                minimum_certainty = cfg.minimum_certainty()
+            if compat_release is None:
+                compat_release = cfg.compat_release()
+                if compat_release:
+                    compat_release = debian_info.codename(
+                        compat_release, default=compat_release)
+            if allow_reformatting is None:
+                allow_reformatting = cfg.allow_reformatting()
+        if minimum_certainty is None:
+            minimum_certainty = DEFAULT_MINIMUM_CERTAINTY
+        if compat_release is None:
             compat_release = debian_info.stable()
+        if allow_reformatting is None:
+            allow_reformatting = False
         with wt.lock_write():
             try:
-                applied, failed = run_lintian_fixers(
+                overall_result = run_lintian_fixers(
                     wt, fixers,
                     update_changelog=args.update_changelog,
                     compat_release=compat_release,
                     verbose=args.verbose,
                     minimum_certainty=args.minimum_certainty,
                     trust_package=args.trust,
-                    allow_reformatting=args.allow_reformatting,
-                    use_inotify=(False if args.disable_inotify else None))
+                    allow_reformatting=allow_reformatting,
+                    use_inotify=(False if args.disable_inotify else None),
+                    subpath=subpath,
+                    net_access=not args.disable_net_access)
             except NotDebianPackage:
                 note("%s: Not a debian package.", wt.basedir)
                 return 1
             except PendingChanges:
                 note("%s: Please commit pending changes first.", wt.basedir)
+                if args.verbose:
+                    from breezy.status import show_tree_status
+                    show_tree_status(wt)
                 return 1
-        if applied:
+        if overall_result.success:
             all_tags = set()
-            for result, summary in applied:
+            for result, summary in overall_result.success:
                 all_tags.update(result.fixed_lintian_tags)
             if all_tags:
                 note("Lintian tags fixed: %r" % all_tags)
@@ -191,9 +228,15 @@ def main(argv=None):
                      "but there are no affected lintian tags.")
         else:
             note("No changes made.")
-        if failed and not args.verbose:
+        if overall_result.failed_fixers and not args.verbose:
             note("Some fixer scripts failed to run: %r. "
-                 "Run with --verbose for details.", set(failed.keys()))
+                 "Run with --verbose for details.",
+                 set(overall_result.failed_fixers.keys()))
+        if overall_result.formatting_unpreservable and not args.verbose:
+            note('Some fixer scripts were unable to preserve formatting: %r. '
+                 'Run with --allow-reformatting to reformat %r.',
+                 set(overall_result.formatting_unpreservable),
+                 set(overall_result.formatting_unpreservable.values()))
         if args.diff:
             from breezy.diff import show_diff_trees
             show_diff_trees(
