@@ -19,12 +19,79 @@
 
 import collections
 from itertools import takewhile
+import os
 
 from debian.changelog import Version
 from debian.deb822 import Deb822
+import subprocess
 
 from ._deb822 import PkgRelation
 from .deb822 import update_deb822
+from .reformatting import GeneratedFile
+
+
+def dh_gnome_clean():
+    """Run the dh_gnome_clean command.
+
+    This needs to do some post-hoc cleaning, since dh_gnome_clean
+    writes various debhelper log files that should not be checked in.
+    """
+    for n in os.listdir('debian'):
+        if n.endswith('.debhelper.log'):
+            raise AssertionError('pre-existing .debhelper.log files')
+    subprocess.check_call(["dh_gnome_clean"])
+    for n in os.listdir('debian'):
+        if n.endswith('.debhelper.log'):
+            os.unlink('debian/' + n)
+
+
+def pg_buildext_updatecontrol():
+    """Run the 'pg_buildext updatecontrol' command.
+    """
+    subprocess.check_call(["pg_buildext", "updatecontrol"])
+
+
+def guess_template_type(template_path):
+    with open(template_path, 'rb') as f:
+        template = f.read()
+        if b'@GNOME_TEAM@' in template:
+            return 'gnome'
+        elif b'@cdbs@' in template:
+            return 'cdbs'
+        elif b'PGVERSION' in template:
+            return 'postgresql'
+        elif b'@lintian-brush-test@' in template:
+            return 'lintian-brush-test'
+        else:
+            deb822 = Deb822(template)
+            build_depends = deb822.get('Build-Depends', '')
+            if any(iter_relations(build_depends, 'gnome-pkg-tools')):
+                return 'gnome'
+            if any(iter_relations(build_depends, 'cdbs')):
+                return 'cdbs'
+    return None
+
+
+def _update_control_template(template_path, path, paragraph_cb):
+    template_type = guess_template_type(template_path)
+    if template_type is None:
+        raise GeneratedFile(path, template_path)
+    if not update_deb822(template_path, paragraph_cb=paragraph_cb):
+        # A bit odd, since there were changes to the output file. Anyway.
+        return False
+    if template_type == 'cdbs':
+        update_deb822(path, paragraph_cb=paragraph_cb, allow_generated=True)
+    elif template_type == 'gnome':
+        dh_gnome_clean()
+    elif template_type == 'postgresql':
+        pg_buildext_updatecontrol()
+    elif template_type == 'lintian-brush-test':
+        with open(template_path, 'rb') as inf, open(path, 'wb') as outf:
+            outf.write(
+                inf.read().replace(b'@lintian-brush-test@', b'testvalue'))
+    else:
+        raise AssertionError
+    return True
 
 
 def update_control(path='debian/control', source_package_cb=None,
@@ -36,8 +103,12 @@ def update_control(path='debian/control', source_package_cb=None,
         else:
             if binary_package_cb is not None:
                 binary_package_cb(paragraph)
-
-    return update_deb822(path, paragraph_cb=paragraph_cb)
+    try:
+        return update_deb822(path, paragraph_cb=paragraph_cb)
+    except GeneratedFile as e:
+        if not e.template_path:
+            raise
+        return _update_control_template(e.template_path, path, paragraph_cb)
 
 
 def parse_relations(text):
@@ -97,19 +168,36 @@ def get_relation(relationstr, package):
       relationstr: package relation string
       package: package name
     Returns:
-      Relation object
+      Tuple with offset and relation object
+    """
+    for offset, relation in iter_relations(relationstr, package):
+        names = [r.name for r in relation]
+        if len(names) > 1 and package in names:
+            raise ValueError("Complex rule for %s , aborting" % package)
+        if names != [package]:
+            continue
+        return offset, relation
+    raise KeyError(package)
+
+
+def iter_relations(relationstr, package):
+    """Iterate over the relations relevant for a particular package.
+
+    Args:
+      relationstr: package relation string
+      package: package name
+    Yields:
+      Tuples with offset and relation objects
     """
     relations = parse_relations(relationstr)
-    for (head_whitespace, relation, tail_whitespace) in relations:
+    for i, (head_whitespace, relation, tail_whitespace) in enumerate(
+            relations):
         if isinstance(relation, str):  # formatting
             continue
         names = [r.name for r in relation]
-        if len(names) > 1 and names[0] == package:
-            raise Exception("Complex rule for %s , aborting" % package)
-        if names != [package]:
+        if package not in names:
             continue
-        return relation
-    raise KeyError(package)
+        yield i, relation
 
 
 def ensure_minimum_version(relationstr, package, minimum_version):
@@ -150,13 +238,14 @@ def ensure_minimum_version(relationstr, package, minimum_version):
     return relationstr
 
 
-def ensure_exact_version(relationstr, package, version):
+def ensure_exact_version(relationstr, package, version, position=None):
     """Update a relation string to depend on a specific version.
 
     Args:
       relationstr: package relation string
       package: package name
       version: Exact version to depend on
+      position: Optional position in the list to insert any new entries
     Returns:
       updated relation string
     """
@@ -182,56 +271,82 @@ def ensure_exact_version(relationstr, package, version):
         changed = True
         _add_dependency(
             relations,
-            [PkgRelation(name=package, version=('=', version))])
+            [PkgRelation(name=package, version=('=', version))],
+            position=position)
     if changed:
         return format_relations(relations)
     # Just return the original; we don't preserve all formatting yet.
     return relationstr
 
 
-def _add_dependency(relations, relation):
+def _add_dependency(relations, relation, position=None):
     """Add a dependency to a depends line.
 
     Args:
       relations: existing list of relations
       relation: New relation
+      position: Optional position to insert the new relation
     Returns:
       Nothing
     """
     if len(relations) == 0:
         head_whitespace = ''
+        tail_whitespace = ''
     elif len(relations) == 1:
         head_whitespace = (relations[0][0] or " ")  # Best guess
-    else:
-        ws = collections.defaultdict(lambda: 0)
-        for r in relations[1:]:
-            ws[r[0]] += 1
-        if len(ws) == 1:
-            head_whitespace = list(ws.keys())[0]
-        else:
-            head_whitespace = relations[-1][0]  # Best guest
-    if len(relations) == 0:
         tail_whitespace = ''
     else:
-        tail_whitespace = relations[-1][2]
-        relations[-1] = relations[-1][:2] + ('', )
+        hws = collections.defaultdict(lambda: 0)
+        for r in relations[1:]:
+            hws[r[0]] += 1
+        if len(hws) == 1:
+            head_whitespace = list(hws.keys())[0]
+        else:
+            head_whitespace = relations[-1][0]  # Best guess
+        tws = collections.defaultdict(lambda: 0)
+        for r in relations[0:-1]:
+            tws[r[2]] += 1
+        if len(tws) == 1:
+            tail_whitespace = list(tws.keys())[0]
+        else:
+            tail_whitespace = relations[0][2]  # Best guess
 
-    relations.append((head_whitespace, relation, tail_whitespace))
+    if position is None:
+        position = len(relations)
+
+    if position < 0 or position > len(relations):
+        raise IndexError('position out of bounds: %r' % position)
+
+    if position == len(relations):
+        if len(relations) == 0:
+            last_tail_whitespace = ''
+        else:
+            last_tail_whitespace = relations[-1][2]
+            relations[-1] = relations[-1][:2] + (tail_whitespace, )
+        relations.append((head_whitespace, relation, last_tail_whitespace))
+    elif position == 0:
+        relations.insert(
+            position, (relations[0][0], relation, tail_whitespace))
+        relations[1] = (head_whitespace, relations[1][1], relations[1][2])
+    else:
+        relations.insert(
+            position, (head_whitespace, relation, tail_whitespace))
 
 
-def add_dependency(relationstr, relation):
+def add_dependency(relationstr, relation, position=None):
     """Add a dependency to a depends line.
 
     Args:
       relationstr: existing relations line
       relation: New relation
+      position: Optional position to insert relation at (defaults to last)
     Returns:
       Nothing
     """
     relations = parse_relations(relationstr)
     if isinstance(relation, str):
         relation = PkgRelation.parse(relation)
-    _add_dependency(relations, relation)
+    _add_dependency(relations, relation, position=position)
     return format_relations(relations)
 
 
@@ -302,7 +417,7 @@ def ensure_minimum_debhelper_version(build_depends, minimum_version):
     """
     minimum_version = Version(minimum_version)
     try:
-        debhelper_compat = get_relation(
+        offset, debhelper_compat = get_relation(
             build_depends, "debhelper-compat")
     except KeyError:
         pass
@@ -321,6 +436,8 @@ def ensure_minimum_debhelper_version(build_depends, minimum_version):
 def delete_from_list(liststr, item_to_delete):
     items = liststr.split(',')
     item_to_delete = item_to_delete.strip()
+    if not item_to_delete:
+        return liststr
     for i, item in enumerate(items):
         if item.strip() == item_to_delete:
             deleted_item = items.pop(i)
@@ -331,14 +448,25 @@ def delete_from_list(liststr, item_to_delete):
                 # second
                 items[i] = head_whitespace + items[i].lstrip()
             elif i == len(items):
-                items[i-1] = items[i-1].rstrip()
+                if i > 1:
+                    items[i-1] = items[i-1].rstrip()
     return ','.join(items)
 
 
+def read_debian_compat_file(path):
+    """Read a debian/compat file.
+
+    Args:
+      path: Path to read from
+    """
+    with open(path, 'r') as f:
+        line = f.readline().split('#', 1)[0]
+        return int(line.strip())
+
+
 def get_debhelper_compat_version():
     try:
-        with open('debian/compat', 'r') as f:
-            return int(f.read().strip())
+        return read_debian_compat_file('debian/compat')
     except FileNotFoundError:
         pass
 
@@ -349,7 +477,7 @@ def get_debhelper_compat_version():
         return None
 
     try:
-        [relation] = get_relation(
+        offset, [relation] = get_relation(
             control.get("Build-Depends", ""), "debhelper-compat")
     except (IndexError, KeyError):
         return None
