@@ -44,6 +44,7 @@ MAINTAINER_EMAIL_MAP = {
     'pkg-php-pear@lists.alioth.debian.org': 'php-team/pear',
     'pkg-mpd-maintainers@lists.alioth.debian.org': 'mpd-team',
     'pkg-cli-apps-team@lists.alioth.debian.org': 'dotnet-team',
+    'pkg-mono-group@lists.alioth.debian.org': 'dotnet-team',
     }
 
 TEAM_NAME_MAP = {
@@ -51,9 +52,11 @@ TEAM_NAME_MAP = {
     'pkg-go': 'go-team',
     'pkg-fonts': 'fonts-team',
     'pkg-javascript': 'js-team',
+    'pkg-java': 'java-team',
     'pkg-mpd': 'mpd-team',
     'pkg-electronics': 'electronics-team',
     'pkg-xfce': 'xfce-team',
+    'pkg-lxc': 'lxc-team',
     'debian-science': 'science-team',
     'pkg-games': 'games-team',
     'pkg-bluetooth': 'bluetooth-team',
@@ -62,6 +65,50 @@ TEAM_NAME_MAP = {
     'pkg-kde': 'qt-kde-team',
     'debian-islamic': 'islamic-team',
     'pkg-lua': 'lua-team',
+    'pkg-xorg': 'xorg-team',
+    'debian-astro': 'debian-astro-team',
+    'pkg-icecast': 'multimedia-team',
+    'glibc-bsd': 'bsd-team',
+    'pkg-nvidia': 'nvidia-team',
+    'pkg-llvm': 'llvm-team',
+    'pkg-nagios': 'nagios-team',
+    'pkg-sugar': 'pkg-sugar-team',
+    'pkg-phototools': 'debian-phototools-team',
+    'pkg-netmeasure': 'ineteng-team',
+    'pkg-hamradio': 'debian-hamradio-team',
+    'pkg-sass': 'sass-team',
+    'pkg-rpm': 'pkg-rpm-team',
+    'tts': 'tts-team',
+    'python-apps': 'python-team/applications',
+    'pkg-monitoring': 'monitoring-team',
+    'pkg-perl': 'perl-team/modules',
+    'debian-iot': 'debian-iot-team',
+    'pkg-bitcoin': 'cryptocoin-team',
+    'pkg-cyrus-imapd': 'debian',
+    'pkg-dns': 'dns-team',
+    'pkg-freeipa': 'freeipa-team',
+    'pkg-ocaml-team': 'ocaml-team',
+    'pkg-vdr-dvb': 'vdr-team',
+    'debian-in': 'debian-in-team',
+    'pkg-octave': 'pkg-octave-team',
+    'pkg-postgresql': 'postgresql',
+    'pkg-grass': 'debian-gis-team',
+    'pkg-evolution': 'gnome-team',
+    'pkg-gnome': 'gnome-team',
+    'pkg-exppsy': 'neurodebian-team',
+    'pkg-voip': 'pkg-voip-team',
+    'pkg-privacy': 'pkg-privacy-team',
+    'pkg-libvirt': 'libvirt-team',
+    'debian-ha': 'ha-team',
+    'debian-lego': 'debian-lego-team',
+    'calendarserver': 'calendarserver-team',
+    '3dprinter': '3dprinting-team',
+    'pkg-multimedia': 'multimedia-team',
+    'pkg-emacsen': 'emacsen-team',
+    'pkg-haskell': 'haskell-team',
+    'pkg-gnutls': 'gnutls-team',
+    'pkg-mysql': 'mariadb-team',
+    'pkg-php': 'php-team',
 }
 
 
@@ -153,4 +200,42 @@ def salsa_url_from_alioth_url(vcs_type, alioth_url):
             return alioth_url.replace(
                 'svn://svn.debian.org/pkg-lua/packages',
                 'https://salsa.debian.org/lua-team')
+        parsed_url = urlparse(alioth_url)
+        if (parsed_url.scheme == 'svn' and
+                parsed_url.netloc in (
+                    ('svn.debian.org', 'anonscm.debian.org'))):
+            parts = parsed_url.path.strip('/').split('/')
+            if parts[0] == 'svn':
+                parts.pop(0)
+            if (len(parts) == 3 and
+                    parts[0] in TEAM_NAME_MAP and
+                    parts[2] == 'trunk'):
+                return 'https://salsa.debian.org/%s/%s' % (
+                    TEAM_NAME_MAP[parts[0]], parts[1])
+            if (len(parts) == 3 and
+                    parts[0] in TEAM_NAME_MAP and
+                    parts[1] == 'trunk'):
+                return 'https://salsa.debian.org/%s/%s' % (
+                    TEAM_NAME_MAP[parts[0]], parts[2])
+            if (len(parts) == 4 and
+                    parts[0] in TEAM_NAME_MAP and
+                    parts[1] == 'packages' and
+                    parts[3] == 'trunk'):
+                return 'https://salsa.debian.org/%s/%s' % (
+                    TEAM_NAME_MAP[parts[0]], parts[2])
+            if (len(parts) == 4 and
+                    parts[0] in TEAM_NAME_MAP and
+                    parts[1] == 'trunk' and parts[2] == 'packages'):
+                return 'https://salsa.debian.org/%s/%s' % (
+                    TEAM_NAME_MAP[parts[0]], parts[3])
+            if (len(parts) > 3 and
+                    parts[0] in TEAM_NAME_MAP and
+                    parts[-2] == 'trunk'):
+                return 'https://salsa.debian.org/%s/%s' % (
+                    TEAM_NAME_MAP[parts[0]], parts[-1])
+            if (len(parts) == 3 and
+                    parts[0] in TEAM_NAME_MAP and
+                    parts[1] == 'packages'):
+                return 'https://salsa.debian.org/%s/%s' % (
+                    TEAM_NAME_MAP[parts[0]], parts[2])
     return None
