@@ -20,6 +20,8 @@ import unittest
 
 def test_suite():
     names = [
+        'changelog',
+        'config',
         'control',
         'copyright',
         'deb822',
