@@ -45,6 +45,7 @@ from lintian_brush import (
     UnsupportedCertainty,
     available_lintian_fixers,
     check_clean_tree,
+    add_changelog_entry,
     certainty_sufficient,
     get_committer,
     increment_version,
@@ -52,6 +53,7 @@ from lintian_brush import (
     parse_script_fixer_output,
     run_lintian_fixer,
     run_lintian_fixers,
+    select_fixers,
     version_string,
     )
 
@@ -157,7 +159,7 @@ Arch: all
 class DummyFixer(Fixer):
 
     def run(self, basedir, current_version, compat_release, minimum_certainty,
-            trust_package, allow_reformatting):
+            trust_package, allow_reformatting, net_access):
         with open(os.path.join(basedir, 'debian/control'), 'a') as f:
             f.write('a new line\n')
         return FixerResult("Fixed some tag.\nExtended description.",
@@ -167,7 +169,7 @@ class DummyFixer(Fixer):
 class FailingFixer(Fixer):
 
     def run(self, basedir, current_version, compat_release, minimum_certainty,
-            trust_package, allow_reformatting):
+            trust_package, allow_reformatting, net_access):
         with open(os.path.join(basedir, 'debian/foo'), 'w') as f:
             f.write("blah")
         with open(os.path.join(basedir, 'debian/control'), 'a') as f:
@@ -225,7 +227,8 @@ Arch: all
 
         class UncertainFixer(Fixer):
             def run(self, basedir, current_version, compat_release,
-                    minimum_certainty, trust_package, allow_reformatting):
+                    minimum_certainty, trust_package, allow_reformatting,
+                    net_access):
                 with open(os.path.join(basedir, 'debian/somefile'), 'w') as f:
                     f.write("test")
                 return FixerResult("Renamed a file.", certainty='possible')
@@ -241,7 +244,8 @@ Arch: all
 
         class UncertainFixer(Fixer):
             def run(self, basedir, current_version, compat_release,
-                    minimum_certainty, trust_package, allow_reformatting):
+                    minimum_certainty, trust_package, allow_reformatting,
+                    net_access):
                 with open(os.path.join(basedir, 'debian/somefile'), 'w') as f:
                     f.write("test")
                 return FixerResult("Renamed a file.", certainty='possible')
@@ -256,7 +260,8 @@ Arch: all
 
         class NewFileFixer(Fixer):
             def run(self, basedir, current_version, compat_release,
-                    minimum_certainty, trust_package, allow_reformatting):
+                    minimum_certainty, trust_package, allow_reformatting,
+                    net_access):
                 with open(os.path.join(basedir, 'debian/somefile'), 'w') as f:
                     f.write("test")
                 return FixerResult("Created new file.", ['some-tag'])
@@ -287,7 +292,8 @@ Arch: all
 
         class RenameFileFixer(Fixer):
             def run(self, basedir, current_version, compat_release,
-                    minimum_certainty, trust_package, allow_reformatting):
+                    minimum_certainty, trust_package, allow_reformatting,
+                    net_access):
                 os.rename(os.path.join(basedir, 'debian/control'),
                           os.path.join(basedir, 'debian/control.blah'))
                 return FixerResult("Renamed a file.")
@@ -316,7 +322,8 @@ Arch: all
 
         class EmptyFixer(Fixer):
             def run(self, basedir, current_version, compat_release,
-                    minimum_certainty, trust_package, allow_reformatting):
+                    minimum_certainty, trust_package, allow_reformatting,
+                    net_access):
                 return FixerResult("I didn't actually change anything.")
         with tree.lock_write():
             self.assertRaises(
@@ -499,7 +506,8 @@ blah (0.1) unstable; urgency=medium
 
     def test_no_changes(self):
         tree = self.make_package_tree()
-        self.assertFalse(only_changes_last_changelog_block(tree))
+        self.assertFalse(only_changes_last_changelog_block(
+            tree, 'debian/changelog'))
 
     def test_other_change(self):
         tree = self.make_package_tree()
@@ -514,7 +522,8 @@ Arch: all
 
 """)])
 
-        self.assertFalse(only_changes_last_changelog_block(tree))
+        self.assertFalse(only_changes_last_changelog_block(
+            tree, 'debian/changelog'))
 
     def test_other_changes(self):
         tree = self.make_package_tree()
@@ -536,7 +545,8 @@ blah (0.1) UNRELEASED; urgency=medium
 
  -- Blah <example@debian.org>  Sat, 13 Oct 2018 11:21:39 +0100
 """)])
-        self.assertFalse(only_changes_last_changelog_block(tree))
+        self.assertFalse(only_changes_last_changelog_block(
+            tree, 'debian/changelog'))
 
     def test_changes_to_other_changelog_entries(self):
         tree = self.make_package_tree()
@@ -554,7 +564,8 @@ blah (0.1) unstable; urgency=medium
 
  -- Blah <example@debian.org>  Sat, 13 Oct 2018 11:21:39 +0100
 """)])
-        self.assertFalse(only_changes_last_changelog_block(tree))
+        self.assertFalse(only_changes_last_changelog_block(
+            tree, 'debian/changelog'))
 
     def test_changes_to_last_only(self):
         tree = self.make_package_tree()
@@ -573,7 +584,8 @@ blah (0.1) unstable; urgency=medium
 
  -- Blah <example@debian.org>  Sat, 13 Oct 2018 11:21:39 +0100
 """)])
-        self.assertTrue(only_changes_last_changelog_block(tree))
+        self.assertTrue(only_changes_last_changelog_block(
+            tree, 'debian/changelog'))
 
     def test_changes_to_last_only_but_released(self):
         tree = self.make_package_tree()
@@ -607,7 +619,8 @@ blah (0.1) unstable; urgency=medium
 
  -- Blah <example@debian.org>  Sat, 13 Oct 2018 11:21:39 +0100
 """)])
-        self.assertFalse(only_changes_last_changelog_block(tree))
+        self.assertFalse(only_changes_last_changelog_block(
+            tree, 'debian/changelog'))
 
 
 class LintianBrushVersion(TestCase):
@@ -761,3 +774,110 @@ class ScriptFixerTests(BaseScriptFixerTests, TestCaseWithTransport):
 class PythonScriptFixerTests(BaseScriptFixerTests, TestCaseWithTransport):
 
     script_fixer_cls = PythonScriptFixer
+
+
+class SelectFixersTests(TestCase):
+
+    def test_exists(self):
+        self.assertEqual(
+            ['dummy1'],
+            [f.name for f in select_fixers(
+                [DummyFixer('dummy1', 'some-tag'),
+                 DummyFixer('dummy2', 'other-tag')],
+                ['dummy1'])])
+
+    def test_missing(self):
+        self.assertRaises(
+            KeyError, select_fixers, [DummyFixer('dummy', 'some-tag')],
+            ['other'])
+
+
+class ChangelogAddEntryTests(TestCaseWithTransport):
+
+    def test_edit_existing(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([
+            ('debian/', ),
+            ('debian/changelog', """\
+lintian-brush (0.35) UNRELEASED; urgency=medium
+
+  * Support updating templated debian/control files that use cdbs
+    template.
+
+ -- Joe Example <joe@example.com>  Fri, 04 Oct 2019 02:36:13 +0000
+ """)])
+        tree.add(['debian', 'debian/changelog'])
+        self.overrideEnv('DEBFULLNAME', 'Jane Example')
+        self.overrideEnv('DEBEMAIL', 'jane@example.com')
+        add_changelog_entry(tree, 'debian/changelog', 'Add a foo')
+        self.assertFileEqual("""\
+lintian-brush (0.35) UNRELEASED; urgency=medium
+
+  [ Joe Example ]
+  * Support updating templated debian/control files that use cdbs
+    template.
+
+  [ Jane Example ]
+  * Add a foo
+
+ -- Joe Example <joe@example.com>  Fri, 04 Oct 2019 02:36:13 +0000
+""", 'debian/changelog')
+
+    def test_add_new(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([
+            ('debian/', ),
+            ('debian/changelog', """\
+lintian-brush (0.35) unstable; urgency=medium
+
+  * Support updating templated debian/control files that use cdbs
+    template.
+
+ -- Joe Example <joe@example.com>  Fri, 04 Oct 2019 02:36:13 +0000
+ """)])
+        tree.add(['debian', 'debian/changelog'])
+        self.overrideEnv('DEBFULLNAME', 'Jane Example')
+        self.overrideEnv('DEBEMAIL', 'jane@example.com')
+        add_changelog_entry(tree, 'debian/changelog', 'Add a foo')
+        text = tree.get_file_text('debian/changelog').decode()
+        lines = text.splitlines()
+        self.assertEqual([
+            'lintian-brush (0.36) UNRELEASED; urgency=medium',
+            '',
+            '  * Add a foo',
+            ''],
+            lines[:4])
+        self.assertTrue(
+            lines[4].startswith(' -- Jane Example <jane@example.com>  '),
+            lines[4])
+
+    def test_edit_broken_first_line(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([
+            ('debian/', ),
+            ('debian/changelog', """\
+THIS IS NOT A PARSEABLE LINE
+lintian-brush (0.35) UNRELEASED; urgency=medium
+
+  * Support updating templated debian/control files that use cdbs
+    template.
+
+ -- Joe Example <joe@example.com>  Fri, 04 Oct 2019 02:36:13 +0000
+ """)])
+        tree.add(['debian', 'debian/changelog'])
+        self.overrideEnv('DEBFULLNAME', 'Jane Example')
+        self.overrideEnv('DEBEMAIL', 'jane@example.com')
+        add_changelog_entry(tree, 'debian/changelog', 'Add a foo')
+        self.assertFileEqual("""\
+THIS IS NOT A PARSEABLE LINE
+lintian-brush (0.35) UNRELEASED; urgency=medium
+
+  [ Joe Example ]
+  * Support updating templated debian/control files that use cdbs
+    template.
+
+  [ Jane Example ]
+  * Add a foo
+
+ -- Joe Example <joe@example.com>  Fri, 04 Oct 2019 02:36:13 +0000
+""", 'debian/changelog')
