@@ -27,6 +27,7 @@ from warnings import warn
 from lintian_brush import USER_AGENT
 from lintian_brush.vcs import (
     sanitize_url as sanitize_vcs_url,
+    probe_vcs_url,
     )
 from lintian_brush.watch import parse_watch_file
 from urllib.request import urlopen, Request
@@ -105,6 +106,14 @@ def get_python_pkg_info(path, trust_package=False):
                 stdout=subprocess.PIPE)
         except FileNotFoundError:
             pass
+        except subprocess.CalledProcessError:
+            if args[0] == 'python3':
+                args[0] = 'python2'
+            else:
+                raise
+            subprocess.call(
+                args, cwd=td, stderr=subprocess.PIPE,
+                stdout=subprocess.PIPE)
         return read_python_pkg_info(td)
 
 
@@ -181,16 +190,25 @@ def guess_from_package_json(path, trust_package):
             if parsed_url.scheme and parsed_url.netloc:
                 yield 'Repository', sanitize_vcs_url(repo_url), 'certain'
             else:
-                # Some people seem to default github. :(
+                # Some people seem to default to github. :(
                 repo_url = 'https://github.com/' + parsed_url.path
                 yield 'Repository', sanitize_vcs_url(repo_url), 'possible'
     if 'bugs' in package:
-        yield 'Bugs-Database', package['bugs'], 'certain'
+        if isinstance(package['bugs'], dict):
+            url = package['bugs'].get('url')
+        else:
+            url = package['bugs']
+        if url:
+            yield 'Bugs-Database', url, 'certain'
 
 
 def guess_from_package_xml(path, trust_package):
     import xml.etree.ElementTree as ET
-    tree = ET.parse(path)
+    try:
+        tree = ET.parse(path)
+    except ET.ParseError as e:
+        warn('Unable to parse package.xml: %s' % e)
+        return
     root = tree.getroot()
     assert root.tag in (
         'package', '{http://pear.php.net/dtd/package-2.0}package',
@@ -273,19 +291,37 @@ def guess_from_debian_copyright(path, trust_package):
 
 def guess_from_readme(path, trust_package):
     import shlex
+    urls = []
     try:
         with open(path, 'rb') as f:
-            for line in f:
-                line = line.decode('utf-8', 'replace')
-                if line.strip().startswith('git clone'):
+            lines = list(f.readlines())
+            for i, line in enumerate(lines):
+                if line.strip().startswith(b'git clone'):
                     line = line.strip()
-                    argv = shlex.split(line)
-                    args = [arg for arg in argv[2:] if not arg.startswith('-')]
-                    url = args[0]
-                    yield ('Repository', sanitize_vcs_url(url), 'possible')
+                    while line.endswith(b'\\'):
+                        line += lines[i+1]
+                        line = line.strip()
+                        i += 1
+                    argv = shlex.split(line.decode('utf-8', 'replace'))
+                    args = [arg for arg in argv[2:]
+                            if not arg.startswith('-') and arg.strip()]
+                    try:
+                        url = args[-2]
+                    except IndexError:
+                        url = args[0]
+                    urls.append(sanitize_vcs_url(url))
     except IsADirectoryError:
         pass
 
+    def prefer_public(url):
+        parsed_url = urlparse(url)
+        if 'ssh' in parsed_url.scheme:
+            return 1
+        return 0
+    urls.sort(key=prefer_public)
+    if urls:
+        yield ('Repository', urls[0], 'possible')
+
 
 def guess_from_meta_json(path, trust_package):
     import json
@@ -316,8 +352,13 @@ def guess_from_meta_yml(path, trust_package):
     specification of the format.
     """
     import ruamel.yaml
+    import ruamel.yaml.reader
     with open(path, 'r') as f:
-        data = ruamel.yaml.load(f, ruamel.yaml.SafeLoader)
+        try:
+            data = ruamel.yaml.load(f, ruamel.yaml.SafeLoader)
+        except ruamel.yaml.reader.ReaderError as e:
+            warn('Unable to parse META.yml: %s' % e)
+            return
         if 'name' in data:
             yield 'Name', data['name'], 'certain'
         if 'resources' in data:
@@ -327,9 +368,12 @@ def guess_from_meta_yml(path, trust_package):
             if 'homepage' in resources:
                 yield 'Homepage', resources['homepage'], 'certain'
             if 'repository' in resources:
-                yield (
-                    'Repository', sanitize_vcs_url(resources['repository']),
-                    'certain')
+                if isinstance(resources['repository'], dict):
+                    url = resources['repository'].get('url')
+                else:
+                    url = resources['repository']
+                if url:
+                    yield ('Repository', sanitize_vcs_url(url), 'certain')
 
 
 def guess_from_doap(path, trust_package):
@@ -491,8 +535,6 @@ def guess_upstream_metadata_items(path, trust_package=False,
                 continue
             yield key, value, certainty
 
-    # TODO(jelmer): validate Repository by querying it somehow?
-
 
 def guess_upstream_metadata(path, trust_package=False):
     """Guess the upstream metadata dictionary.
@@ -525,6 +567,20 @@ def extend_upstream_metadata(code, certainty):
     # TODO(jelmer): Try deriving bug-database too?
 
 
+def check_upstream_metadata(code, certainty):
+    """Check upstream metadata.
+
+    This will make network connections, etc.
+    """
+    if 'Repository' in code and certainty['Repository'] == 'possible':
+        if probe_vcs_url(code['Repository']):
+            certainty['Repository'] = 'certain'
+        else:
+            # TODO(jelmer): Remove altogether, or downgrade to a lesser
+            # certainty?
+            pass
+
+
 if __name__ == '__main__':
     import argparse
     import sys
