@@ -17,7 +17,7 @@
 
 """Utility functions for dealing with Vcs URLs of various types."""
 
-__all__ = ['fixup_broken_git_url']
+__all__ = ['fixup_broken_git_url', 'sanitize_url']
 
 from .salsa import determine_browser_url as determine_salsa_browser_url
 
@@ -28,7 +28,7 @@ def sanitize_url(url):
     url = url.strip()
     if url.startswith('git+http:') or url.startswith('git+https:'):
         return url[4:]
-    return url
+    return fixup_broken_git_url(url)
 
 
 def fix_path_in_port(parsed, branch):
@@ -68,6 +68,12 @@ def fix_salsa_tree_in_url(parsed, branch):
     return None, None
 
 
+def fix_double_slash(parsed, branch):
+    if parsed.path.startswith('//'):
+        return parsed._replace(path=parsed.path[1:]), branch
+    return None, None
+
+
 def fixup_broken_git_url(url):
     """Attempt to fix up broken Git URLs.
 
@@ -81,7 +87,7 @@ def fixup_broken_git_url(url):
     parsed = urlparse(repo_url)
     changed = False
     for fn in [fix_path_in_port, fix_salsa_scheme, fix_salsa_cgit_url,
-               fix_salsa_tree_in_url]:
+               fix_salsa_tree_in_url, fix_double_slash]:
         newparsed, newbranch = fn(parsed, branch)
         if newparsed:
             changed = True
@@ -97,6 +103,11 @@ def fixup_broken_git_url(url):
 
 
 def probe_vcs_url(url):
+    parsed = urlparse(url)
+    # TODO(jelmer): Disable authentication prompting.
+    if parsed.scheme in ('git+ssh', 'ssh', 'bzr+ssh'):
+        # Let's not probe anything possibly non-public.
+        return None
     from breezy.branch import Branch
     try:
         Branch.open(url).last_revision()
