@@ -28,6 +28,26 @@ from lintian_brush.reformatting import (
 
 DEFAULT_VERSION = 4
 
+# TODO(jelmer): Add support for substitution variables:
+
+SUBSTITUTIONS = {
+    # This is substituted with the source package name found in the first line
+    # of the debian/changelog file.
+    '@PACKAGE': None,
+    # This is substituted by the legal upstream version regex (capturing).
+    '@ANY_VERSION@': r'[-_]?(\d[\-+\.:\~\da-zA-Z]*)',
+    # This is substituted by the typical archive file extension regex
+    # (non-capturing).
+    '@ARCHIVE_EXT@': r'(?i)\.(?:tar\.xz|tar\.bz2|tar\.gz|zip|tgz|tbz|txz)',
+    # This is substituted by the typical signature file extension regex
+    # (non-capturing).
+    '@SIGNATURE_EXT@':
+        r'(?i)\.(?:tar\.xz|tar\.bz2|tar\.gz|zip|tgz|tbz|txz)'
+        r'\.(?:asc|pgp|gpg|sig|sign)',
+    # This is substituted by the typical Debian extension regexp (capturing).
+    '@DEB_EXT@': r'[\+~](debian|dfsg|ds|deb)(\.)?(\d+)?$',
+}
+
 
 class WatchFile(object):
 
